using ArgParse

function parse_commandline()
   args = ArgParseSettings()

    @add_arg_table args begin
    "--tol"
        help = "set numerical tolerance for the SDP solver"
        arg_type = Float64
        default = 1e-6
    "--iterations"
        help = "set maximal number of iterations for the SDP solver"
        arg_type = Int
        default = 50000
    "--upper-bound"
        help = "Set an upper bound for the spectral gap"
        arg_type = Float64
        default = Inf
    "--cpus"
        help = "Set number of cpus used by solver (default: auto)"
        arg_type = Int
        required = false
    "--radius"
        help = "Radius of ball B_r(e,S) to find solution over"
        arg_type = Int
        default = 2
    "--warmstart"
        help = "Use warmstart.jl as the initial guess for SCS"
        action = :store_true
    "--MCG"
        help = "Compute for mapping class group of surface of genus N"
        arg_type = Int
        required = false
    "--Higman"
        help = "Compute for Higman Group"
        action = :store_true
    "--Caprace"
        help = "Compute for Higman Group"
        action = :store_true
    end

    return parse_args(args)
end
const PARSEDARGS = parse_commandline()

include("CPUselect.jl")
set_parallel_mthread(PARSEDARGS, workers=false)

include("main.jl")
include("FPGroups_GAP.jl")

if PARSEDARGS["Caprace"]
    G = PropertyTGroups.CapraceGroup(PARSEDARGS)
elseif PARSEDARGS["Higman"]
    G = PropertyTGroups.HigmanGroup(PARSEDARGS)
elseif PARSEDARGS["MCG"] != nothing
    G = PropertyTGroups.MappingClassGroup(PARSEDARGS)
else
    throw("You need to specify one of the --Higman, --Caprace, --MCG N")
end

main(G)
